// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@infoscreen.com, or by
// paper mail at:
//
// Tim Freeman
// 655 S. FairOaks Ave., Apt B-316
// Sunnyvale, CA 94086
//

#ifndef __HashtableIteratorTypeless_h__
#define __HashtableIteratorTypeless_h__

#ifndef __SP_h__
#include "SP.h"
#endif

#ifndef __HashTableTypeless_h__
#include "HashTableTypeless.h"
#endif

// Can't simulataneously insulate the use from modifications to the
// hashtable implementation and provide a copyable iterator construct.
// Compilation is fast, so give up on the insulation. 
class HashTableIteratorTypeless {
  int m_index;
  const SP<const HashTableTypeless> m_table;
  // Only HashTableImpl can make one of us, in its iterator() method.
  friend class HashTableImpl;
 public:
  inline void first () {
    m_table->iteratorFirst (*this);
  }
 private:
  inline HashTableIteratorTypeless (const HashTableTypeless *table)
    : m_table (table)
  {
    first ();
  };
 public:
  void next () {
    m_table->iteratorNext (*this);
  }
  bool isDone () const {
    return m_table->iteratorIsDone (*this);
  }
  // currentItem will assert that isDone is false.
  const Hashable *currentItem () const {
    return m_table->iteratorCurrentItem (*this);
  }
};

#endif
