// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@infoscreen.com, or by
// paper mail at:
//
// Tim Freeman
// 655 S. FairOaks Ave., Apt B-316
// Sunnyvale, CA 94086
//

#ifndef __PositiveSlotValue_h__
#define __PositiveSlotValue_h__

#ifndef __SP_h__
#include "SP.h"
#endif

#ifndef __MemoryUtil_h__
#include "MemoryUtil.h"
#endif

// A slot that contains an integer value that is constrained to be positive.
template <class SubSlot, class T> class PositiveSlotValue : public SubSlot
{
 public:
  PositiveSlotValue (T value)
    : SubSlot (value, "Positive "+SubSlot::getDefaultTypeName())
  {
    assert (value > 0);
  };
  bool hasTheSameTypeAs (const SlotValue *s) const {
    return 0 != dynamic_cast <const PositiveSlotValue<SubSlot, T> *> (s);
  };
  SP<BadSlotValue> PositiveSlotValue::constrain () {
    if (getValue() <= 0) {
      return NEW (BadSlotValue ("positive"));
    } else {
      return 0;
    }
  }
  SP<SlotValue> copy () const {
    return NEW (PositiveSlotValue (*this));
  };
  static void setSlot (Configuration *a_c,
		       const String &name,
		       T value) {
    SP<Configuration> c = a_c;
    assert (value > 0);
    SP<BadSlotValue> b = c->setSlot (name, NEW (PositiveSlotValue (value)));
    assert (!b);
  }
  static T getSlot (const Configuration *a_c,
		    const String &name) {
    return SubSlot::getSlot (a_c, name);
  }
};

#endif
