// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@infoscreen.com, or by
// paper mail at:
//
// Tim Freeman
// 655 S. FairOaks Ave., Apt B-316
// Sunnyvale, CA 94086
//

#ifndef __Refcount_h__
#define __Refcount_h__

// Inherit from this to get a reference count.  Cooperates with SP.h.
// Needs a virtual destructor, so we can pass around a pointer to a
// Refcount and have it mean the entire object.
class Refcount {
  mutable int m_count;
 public:
  Refcount ();
  // When you copy these, the copy isn't automatically referenced.
  Refcount (const Refcount &r) : m_count (0) {
    (void) r; // r is not used, and that's okay.
  }
  virtual ~Refcount ();
  void ref () const;
  void deref () const;
  int count () const;
};

#endif
